<?php

$host = "0.0.0.0";
$port = 7788;
$logFile = __DIR__ . "/timmy_data.log";

$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
socket_set_option($socket, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($socket, $host, $port) or die("Could not bind to socket\n");
socket_listen($socket);

echo "Listening for WebSocket connections on $host:$port...\n";

function decodeWebSocketFrame($data) {
    $payloadLength = ord($data[1]) & 127;
    $maskStart = $payloadLength === 126 ? 4 : ($payloadLength === 127 ? 10 : 2);
    $mask = substr($data, $maskStart, 4);
    $payloadStart = $maskStart + 4;
    $payload = substr($data, $payloadStart);

    $decoded = '';
    for ($i = 0; $i < strlen($payload); $i++) {
        $decoded .= $payload[$i] ^ $mask[$i % 4];
    }

    return $decoded;
}

function encodeWebSocketFrame($message) {
    $header = chr(129); // FIN + text frame
    $length = strlen($message);

    if ($length <= 125) {
        $header .= chr($length);
    } elseif ($length <= 65535) {
        $header .= chr(126) . pack("n", $length);
    } else {
        $header .= chr(127) . pack("Q", $length);
    }

    return $header . $message;
}

date_default_timezone_set('Asia/Dhaka');

while (true) {
    $client = @socket_accept($socket);
    if ($client) {
        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] New client connected.\n", FILE_APPEND);

        $request = socket_read($client, 2048);
        if (preg_match("/Sec-WebSocket-Key: (.*)\r\n/", $request, $matches)) {
            $key = trim($matches[1]);
            $acceptKey = base64_encode(pack('H*', sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
            $headers = "HTTP/1.1 101 Switching Protocols\r\n";
            $headers .= "Upgrade: websocket\r\n";
            $headers .= "Connection: Upgrade\r\n";
            $headers .= "Sec-WebSocket-Accept: $acceptKey\r\n\r\n";
            socket_write($client, $headers, strlen($headers));
        }

        $lastActivity = time();

        while (true) {
            $data = @socket_read($client, 2048, PHP_BINARY_READ);

            if ($data === false || $data === '') {
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Client disconnected.\n", FILE_APPEND);
                @socket_close($client);
                break;
            }

            $decodedData = decodeWebSocketFrame($data);
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Received: $decodedData\n", FILE_APPEND);

            // Respond to heartbeat messages
            $parsedData = json_decode($decodedData, true);
            if ($parsedData && isset($parsedData['cmd']) && $parsedData['cmd'] === 'heartbeat') {
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Heartbeat received.\n", FILE_APPEND);
                $response = json_encode(["cmd" => "heartbeat_ack", "time" => date("Y-m-d H:i:s")]);
                $encodedResponse = encodeWebSocketFrame($response);
                @socket_write($client, $encodedResponse, strlen($encodedResponse));
                continue; // Skip other processing for heartbeat
            }
            
            // Regular command handling
            if ($parsedData && isset($parsedData['cmd'])) {
                // Process registration, attendance, etc. (your existing logic)
                if ($parsedData['cmd'] === 'reg') {
                    $response = json_encode([
                        "ret" => "reg",
                        "result" => true,
                        "cloudtime" => date("Y-m-d H:i:s"),
                        "nosenduser" => true
                    ]);
                } elseif ($parsedData['cmd'] === 'sendlog') {
                    
                    // Log attendance data
                    if (isset($parsedData['record']) && is_array($parsedData['record'])) {
                        foreach ($parsedData['record'] as $log) {
                            // Log data to the file
                            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Attendance Log: " . json_encode($log, JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                    
                            // Extract data from the log
                            $sn = $parsedData['sn'];
                            $user_id = $log['enrollid'];
                            $attendance_datetime = $log['time'];
                            $attendance_date = date('Y-m-d', strtotime($attendance_datetime));
                            $attendance_time = date('H:i:s', strtotime($attendance_datetime));
                            $extra = 0; // Assign null as per requirements
                            $punch_type = $log['mode'];
                            $status_1 = $log['inout'];
                            $status_2 = $log['event'];
                    
                            // Database connection
                            $connection = new mysqli("localhost", "hostzil_edukit", "hostzil_edukit", "hostzil_edukit");
                            if ($connection->connect_error) {
                                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Database connection failed: " . $connection->connect_error . PHP_EOL, FILE_APPEND);
                                continue;
                            }
                    
                            // Prepare the SQL query to insert data into the database
                            $query = "INSERT INTO attendance_log (SN, user_id, attendance_date, attendance_time, extra, punch_type, status_1, status_2) 
                                      VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
                            
                            $stmt = $connection->prepare($query);
                            if ($stmt) {
                                $stmt->bind_param("ssssiiii", $sn, $user_id, $attendance_date, $attendance_time, $extra, $punch_type, $status_1, $status_2);
                                if (!$stmt->execute()) {
                                    file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Database insert failed: " . $stmt->error . PHP_EOL, FILE_APPEND);
                                }
                                $stmt->close();
                            } else {
                                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Failed to prepare SQL statement: " . $connection->error . PHP_EOL, FILE_APPEND);
                            }
                    
                            $connection->close();
                        }
                    }
                    
                    $response = json_encode([
                        "ret" => "sendlog",
                        "result" => true,
                        "count" => $parsedData['count'] ?? 0,
                        "logindex" => $parsedData['logindex'] ?? 0,
                        "cloudtime" => date("Y-m-d H:i:s"),
                        "access" => 1,
                        "message" => "Logs successfully received."
                    ]);
                }

                if (!empty($response)) {
                    $encodedResponse = encodeWebSocketFrame($response);
                    @socket_write($client, $encodedResponse, strlen($encodedResponse));
                }
            }
        }
    }
}
socket_close($socket);

?>


<?php
/*
$host = "0.0.0.0";
$port = 7788;
$logFile = __DIR__ . "/timmy_data.log";
$pingInterval = 10; // Interval to send ping in seconds

// Create and configure the socket
$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
socket_set_option($socket, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($socket, $host, $port) or die("Could not bind to socket\n");
socket_listen($socket);

echo "Listening for WebSocket connections on $host:$port...\n";

function decodeWebSocketFrame($data) {
    $payloadLength = ord($data[1]) & 127;
    $maskStart = $payloadLength === 126 ? 4 : ($payloadLength === 127 ? 10 : 2);
    $mask = substr($data, $maskStart, 4);
    $payloadStart = $maskStart + 4;
    $payload = substr($data, $payloadStart);

    $decoded = '';
    for ($i = 0; $i < strlen($payload); $i++) {
        $decoded .= $payload[$i] ^ $mask[$i % 4];
    }

    return $decoded;
}

function encodeWebSocketFrame($message) {
    $header = chr(129); // FIN + text frame
    $length = strlen($message);

    if ($length <= 125) {
        $header .= chr($length);
    } elseif ($length <= 65535) {
        $header .= chr(126) . pack("n", $length);
    } else {
        $header .= chr(127) . pack("Q", $length);
    }

    return $header . $message;
}

function sendPing($client) {
    $pingFrame = chr(137) . chr(0); // FIN + Ping frame
    @socket_write($client, $pingFrame);
}

while (true) {
    $client = @socket_accept($socket);
    if ($client) {
        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] New client connected.\n", FILE_APPEND);

        // Perform WebSocket handshake
        $request = socket_read($client, 2048);
        if (preg_match("/Sec-WebSocket-Key: (.*)\r\n/", $request, $matches)) {
            $key = trim($matches[1]);
            $acceptKey = base64_encode(pack('H*', sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
            $headers = "HTTP/1.1 101 Switching Protocols\r\n";
            $headers .= "Upgrade: websocket\r\n";
            $headers .= "Connection: Upgrade\r\n";
            $headers .= "Sec-WebSocket-Accept: $acceptKey\r\n\r\n";
            socket_write($client, $headers, strlen($headers));
        }

        $lastActivity = time();

        while (true) {
            $data = @socket_read($client, 2048, PHP_BINARY_READ);
            if ($data === false || $data === '') {
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Client disconnected.\n", FILE_APPEND);
                break;
            }

            $decodedData = decodeWebSocketFrame($data);
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Received: $decodedData\n", FILE_APPEND);

            // Send ping if necessary
            if (time() - $lastActivity > $pingInterval) {
                sendPing($client);
                $lastActivity = time();
            }

            $parsedData = json_decode($decodedData, true);
            if ($parsedData && isset($parsedData['cmd'])) {
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Command received: {$parsedData['cmd']}\n", FILE_APPEND);

                $response = '';
                if ($parsedData['cmd'] === 'reg') {
                    if (isset($parsedData['devinfo'])) {
                        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Device Info: " . json_encode($parsedData['devinfo'], JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                    }

                    $response = json_encode([
                        "ret" => "reg",
                        "result" => true,
                        "cloudtime" => date("Y-m-d H:i:s"),
                        "nosenduser" => true
                    ]);
                } elseif ($parsedData['cmd'] === 'sendlog') {
                    if (isset($parsedData['record']) && is_array($parsedData['record'])) {
                        foreach ($parsedData['record'] as $log) {
                            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Attendance Log: " . json_encode($log, JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                        }
                    }

                    $response = json_encode([
                        "ret" => "sendlog",
                        "result" => true,
                        "count" => $parsedData['count'] ?? 0,
                        "logindex" => $parsedData['logindex'] ?? 0,
                        "cloudtime" => date("Y-m-d H:i:s"),
                        "access" => 1,
                        "message" => "Logs successfully received."
                    ]);
                }

                if ($response) {
                    $encodedResponse = encodeWebSocketFrame($response);
                    socket_write($client, $encodedResponse, strlen($encodedResponse));
                }
            }
        }
        @socket_close($client);
    }
}
socket_close($socket);
*/
?>



<?php
/*
$host = "0.0.0.0";
$port = 7788;
$logFile = __DIR__ . "/timmy_data.log";

$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
// Set socket option to allow address reuse
socket_set_option($socket, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($socket, $host, $port) or die("Could not bind to socket\n");
socket_listen($socket);

echo "Listening for WebSocket connections on $host:$port...\n";

function decodeWebSocketFrame($data)
{
    $payloadLength = ord($data[1]) & 127;
    $maskStart = $payloadLength === 126 ? 4 : ($payloadLength === 127 ? 10 : 2);
    $mask = substr($data, $maskStart, 4);
    $payloadStart = $maskStart + 4;
    $payload = substr($data, $payloadStart);

    $decoded = '';
    for ($i = 0; $i < strlen($payload); $i++) {
        $decoded .= $payload[$i] ^ $mask[$i % 4];
    }

    return $decoded;
}

function encodeWebSocketFrame($message)
{
    $header = chr(129); // FIN + text frame
    $length = strlen($message);

    if ($length <= 125) {
        $header .= chr($length);
    } elseif ($length <= 65535) {
        $header .= chr(126) . pack("n", $length);
    } else {
        $header .= chr(127) . pack("Q", $length);
    }

    return $header . $message;
}

while (true) {
    $client = socket_accept($socket);
    if (!$client) {
        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Failed to accept connection\n", FILE_APPEND);
        continue;
    }

    // Perform WebSocket handshake
    $request = socket_read($client, 2048);
    if (preg_match("/Sec-WebSocket-Key: (.*)\r\n/", $request, $matches)) {
        $key = trim($matches[1]);
        $acceptKey = base64_encode(pack('H*', sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
        $headers = "HTTP/1.1 101 Switching Protocols\r\n";
        $headers .= "Upgrade: websocket\r\n";
        $headers .= "Connection: Upgrade\r\n";
        $headers .= "Sec-WebSocket-Accept: $acceptKey\r\n\r\n";
        socket_write($client, $headers, strlen($headers));
    }

    file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Client connected\n", FILE_APPEND);

    // Handle client in a persistent loop
    while (true) {
        $data = @socket_read($client, 2048, PHP_BINARY_READ);
        if ($data === false || $data === '') {
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Client disconnected\n", FILE_APPEND);
            socket_close($client);
            break;
        }

        $decodedData = decodeWebSocketFrame($data);

        // Log received data
        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Received: " . $decodedData . PHP_EOL, FILE_APPEND);

        $response = '';
        $parsedData = json_decode($decodedData, true);

        if ($parsedData && isset($parsedData['cmd'])) {
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Command received: " . $parsedData['cmd'] . PHP_EOL, FILE_APPEND);

            if ($parsedData['cmd'] === 'reg') {
                // Respond to registration success
                $response = json_encode([
                    "ret" => "reg",
                    "result" => true,
                    "cloudtime" => date("Y-m-d H:i:s"),
                    "nosenduser" => true
                ]);
            } elseif ($parsedData['cmd'] === 'sendlog') {
                // Log attendance data
                if (isset($parsedData['record']) && is_array($parsedData['record'])) {
                    foreach ($parsedData['record'] as $log) {
                        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Attendance Log: " . json_encode($log, JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                    }
                }

                // Respond to log success
                $response = json_encode([
                    "ret" => "sendlog",
                    "result" => true,
                    "count" => $parsedData['count'] ?? 0,
                    "logindex" => $parsedData['logindex'] ?? 0,
                    "cloudtime" => date("Y-m-d H:i:s"),
                    "access" => 1,
                    "message" => "Logs successfully received."
                ]);
            } else {
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Unknown command: " . $parsedData['cmd'] . PHP_EOL, FILE_APPEND);
            }
        } else {
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Invalid or malformed data\n", FILE_APPEND);
        }

        // Send response if needed
        if ($response) {
            $encodedResponse = encodeWebSocketFrame($response);
            socket_write($client, $encodedResponse, strlen($encodedResponse));
        }
    }
}
*/
?>


<?php
/*
$host = "0.0.0.0";
$port = 7788;
$logFile = __DIR__ . "/timmy_data.log";

$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
// Set socket option to allow address reuse
socket_set_option($socket, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($socket, $host, $port) or die("Could not bind to socket\n");
socket_listen($socket);

echo "Listening for WebSocket connections on $host:$port...\n";

function decodeWebSocketFrame($data)
{
    $payloadLength = ord($data[1]) & 127;
    $maskStart = $payloadLength === 126 ? 4 : ($payloadLength === 127 ? 10 : 2);
    $mask = substr($data, $maskStart, 4);
    $payloadStart = $maskStart + 4;
    $payload = substr($data, $payloadStart);

    $decoded = '';
    for ($i = 0; $i < strlen($payload); $i++) {
        $decoded .= $payload[$i] ^ $mask[$i % 4];
    }

    return $decoded;
}

function encodeWebSocketFrame($message)
{
    $header = chr(129); // FIN + text frame
    $length = strlen($message);

    if ($length <= 125) {
        $header .= chr($length);
    } elseif ($length <= 65535) {
        $header .= chr(126) . pack("n", $length);
    } else {
        $header .= chr(127) . pack("Q", $length);
    }

    return $header . $message;
}

while (true) {
    $client = socket_accept($socket);

    // Perform WebSocket handshake
    $request = socket_read($client, 2048);
    if (preg_match("/Sec-WebSocket-Key: (.*)\r\n/", $request, $matches)) {
        $key = trim($matches[1]);
        $acceptKey = base64_encode(pack('H*', sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
        $headers = "HTTP/1.1 101 Switching Protocols\r\n";
        $headers .= "Upgrade: websocket\r\n";
        $headers .= "Connection: Upgrade\r\n";
        $headers .= "Sec-WebSocket-Accept: $acceptKey\r\n\r\n";
        socket_write($client, $headers, strlen($headers));
    }

    while (true) {
        // Read and decode WebSocket frame
        $data = socket_read($client, 2048);
        if (!$data) {
            socket_close($client);
            break;
        }

        $decodedData = decodeWebSocketFrame($data);

        // Log all received data
        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] " . $decodedData . PHP_EOL, FILE_APPEND);

        $response = '';
        $parsedData = json_decode($decodedData, true);

        if ($parsedData && isset($parsedData['cmd'])) {
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Command received: " . $parsedData['cmd'] . PHP_EOL, FILE_APPEND);

            if ($parsedData['cmd'] === 'reg') {
                // Log registration data
                if (isset($parsedData['devinfo'])) {
                    file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Device Info: " . json_encode($parsedData['devinfo'], JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                }

                // Respond to registration success
                $response = json_encode([
                    "ret" => "reg",
                    "result" => true,
                    "cloudtime" => date("Y-m-d H:i:s"),
                    "nosenduser" => true // Indicate no need to resend user data
                ]);
            } elseif ($parsedData['cmd'] === 'sendlog') {
                // Log attendance data
                if (isset($parsedData['record']) && is_array($parsedData['record'])) {
                    foreach ($parsedData['record'] as $log) {
                        file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Attendance Log: " . json_encode($log, JSON_PRETTY_PRINT) . PHP_EOL, FILE_APPEND);
                    }
                }

                // Respond to log success
                $response = json_encode([
                    "ret" => "sendlog",
                    "result" => true,
                    "count" => $parsedData['count'] ?? 0,
                    "logindex" => $parsedData['logindex'] ?? 0,
                    "cloudtime" => date("Y-m-d H:i:s"),
                    "access" => 1, // Optional: 1 to grant access, 0 to deny
                    "message" => "Logs successfully received."
                ]);
            } else {
                // Handle unknown command
                file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Unknown command: " . $parsedData['cmd'] . PHP_EOL, FILE_APPEND);
            }
        }

        // Send WebSocket frame back to the client
        if ($response) {
            $encodedResponse = encodeWebSocketFrame($response);
            socket_write($client, $encodedResponse, strlen($encodedResponse));
        }
    }
}
*/
?>
